--[[
SimpleMD, an addon by Starinnia

This addon started as a simple text-warning system for use in my guild.
I released it to wowace and Curse in the hopes it would help other hunters.

Cooldown bars were recently requested and implemented.
Rather than re-invent the wheel, I looked at other addons for a useful bar implementation,
MendWatch by Polima was the source of the bars used in this addon.
]]

-- if player class is not hunter do not load anything
if (select(2, UnitClass("player"))) ~= "HUNTER" then return end

local L = AceLibrary("AceLocale-2.2"):new("SimpleMD")
local PlayerName = UnitName("player")
-- to be used soon because the hunters in my guild are lazy and I forsee version checks being needed
local revision = "$Revision: 46732 $"

SimpleMD = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceComm-2.0", "Sink-1.0", "CandyBar-2.0")
local DD = AceLibrary("Dewdrop-2.0")
local colors = AceLibrary("PaintChips-2.0")

function SimpleMD:OnInitialize()	
	self.textures = {
		["default"] = nil,
	}
	
	local defaults = {
		messageGain = L["<player> cast Misdirection on <target>"],
		messageFade = L["Misdirection fades from <target>"],
		r = 1.0,
		g = 1.0,
		b = 1.0,
		channel = "",
		showInChannel = false,
		showInChat = false,
		showOnScreen = true,
		showFloating = false,
		showSelf = true,
		showRW = false,
		showBars = true,
		showGain = true,
		showFade = true,
		ready = true,
		ten = true,
		thirty = true,
		sixty = false,
		texture = "default",
		barR = 1.0,
		barG = 1.0,
		barB = 1.0,
		BGr = 1.0,
		BGg = 1.0,
		BGb = 1.0,
		txtR = 1.0,
		txtG = 1.0,
		txtB = 1.0,
		txtColor = "white",
		bg_alpha = 1.0,
		textSize = 11.08,
		barScale = 1.0,
		barWidth = 195.00,
		barHeight = 17.00,
		growUp = false,
		reverse = false,
		position = {},
	}
	
	SimpleMD:RegisterDB("SimplemdDB")
	SimpleMD:RegisterDefaults('profile', defaults)
	SimpleMD:SetCommPrefix("SimpleMD")
	
	self.anchor = self:CreateAnchor("SimpleMD", 0, 1, 0)
	DD:Register(self.anchor, "children", self.MenuRequest, 'cursorX', true, 'cursorY', true)
	
	self:RegisterCandyBarGroup("SimpleMD")
	self:SetCandyBarGroupPoint("SimpleMD", "TOP", self.anchor, "BOTTOM", 0, 0)
	self:SetCandyBarGroupGrowth("SimpleMD", self.db.profile.growUp)
	
	self.count = 0
	self:UpdateColors()
end

function SimpleMD:OnEnable()
	self.tex = self:GetTexture(L["Misdirection"])
	
	self.bars = {}
	for i = 1, 20 do
		self.bars[i] = {} 
	end
	
	self:RegisterComm("SimpleMD", "GROUP", "OnCommReceive")
	
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD")
end

function SimpleMD:OnDisable()
	if self.bars then
		self:RemoveBars()
		self.bars = nil
	end
	self:UnregisrerAllEvents()
end

function SimpleMD:OnCommReceive(prefix, sender, distribution, message)	
	if self.db.profile.showInChat and message ~= "GAIN" and message ~= "READINESS" then
		self:Print(message)
	end
	if self.db.profile.showOnScreen and message ~= "GAIN" and message ~= "READINESS" then
		UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
	end
	if self.db.profile.showFloating and message ~= "GAIN" and message ~= "READINESS" then
		self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
	end
	if message == "GAIN" and self.db.profile.showBars then
		self:StartTimer(sender)
	end
	if message == "READINESS" and self.db.profile.showBars then
		local idnum = sender.."1"
		self:StopCandyBar(idnum)
	end
end

function SimpleMD:SendMessage(message, id)	
	SimpleMD:SendCommMessage("GROUP", message)
	
	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChannel and self.db.profile.channel ~= "" then
			SendChatMessage(message, "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
		end
	
		if self.db.profile.showFloating and self.db.profile.showSelf and id ~= 1 then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
	end
end

--check to see if spell is Misdirection, if it is set the spell target
function SimpleMD:UNIT_SPELLCAST_SENT(arg1, arg2, arg3, arg4)
	if arg1 == "player" then
		if arg2 == L["Misdirection"] then
			self.SpellTarget = arg4
		end
	end
end

-- if spellcast succeeds call the gain function and schedule cooldown events
-- needed to make sure messages are not sent when target is out of LOS or out of range or invalid
function SimpleMD:UNIT_SPELLCAST_SUCCEEDED(arg1, arg2, arg3)
	if arg1 == "player" then
		if arg2 == L["Misdirection"] then
			self:Make_CD_Bar()
			if self.db.profile.showGain then
				self:Gain()
			end
			if self.db.profile.sixty then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 60, 60) --60 second warning
			end
			if self.db.profile.thirty then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 90, 30) --30 second warning
			end
			if self.db.profile.ten then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 110, 10) -- 10 second warning
			end
			if self.db.profile.ready then
			self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 120, 0) -- cooldown finished
			end
		end
		--account for the readiness talent
		if arg2 == L["Readiness"] then
			self:CancelAllScheduledEvents()
			local temp = PlayerName .. " " .. L["used Readiness! MD ready!"]
			self:SendMessage(temp)
			SimpleMD:SendCommMessage("GROUP", "READINESS")
			if self.db.profile.showSelf and self.db.profile.showBars then
				local idnum = PlayerName.."1"
				self:StopCandyBar(idnum)
			end
		end
	end
end

function SimpleMD:CHAT_MSG_SPELL_AURA_GONE_SELF(arg1)
	if arg1 == L["Misdirection fades from you."] then
		if self.db.profile.showFade then
			self:Fade()
		end
	end
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD(arg1)
	if arg1 == 60 then
		local temp = PlayerName .. " " .. L["has 60 seconds left on MD cooldown."]
		self:SendMessage(temp, 2)
	end
	if arg1 == 30 then
		local temp = PlayerName .. " " .. L["has 30 seconds left on MD cooldown."]
		self:SendMessage(temp, 2)
	end
	if arg1 == 10 then
		local temp = PlayerName .. " " .. L["has 10 seconds left on MD cooldown!"]
		self:SendMessage(temp, 2)
	end
	if arg1 == 0 then
		local temp = PlayerName .. " " .. L["has MD ready!"]
		self:SendMessage(temp, 2)
	end
end

-- called on successful MD casts
function SimpleMD:Gain()
	local message = string.gsub(self.db.profile.messageGain, "<player>", PlayerName)
	message = string.gsub(message, "<target>", self.SpellTarget)

	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChat and self.db.profile.showSelf then
			self:Print(message)
		end
		if self.db.profile.showOnScreen and self.db.profile.showSelf then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
		if self.db.profile.showFloating and self.db.profile.showSelf then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
		if (IsRaidLeader() or IsRaidOfficer()) and self.db.profile.showRW then
			SendChatMessage(message, "RAID_WARNING")
		end
		
		self:SendMessage(message, 1)
	end
end

function SimpleMD:Make_CD_Bar()
	self:SendCommMessage("GROUP", "GAIN")
	if self.db.profile.showSelf and self.db.profile.showBars then
		self:StartTimer(PlayerName)
	end
end

-- called on Misdirection fading from self
function SimpleMD:Fade()
	local message = string.gsub(self.db.profile.messageFade, "<player>", PlayerName)
	message = string.gsub(message, "<target>", self.SpellTarget)

	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChat and self.db.profile.showSelf then
			self:Print(message)
		end
		if self.db.profile.showOnScreen and self.db.profile.showSelf then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
		if self.db.profile.showFloating and self.db.profile.showSelf then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
		
		self:SendMessage(message, 1)
	end
end

-- test broadcast for making sure colors or channel is correct
function SimpleMD:Test()
	if self.db.profile.showInChat then
		self:Print(L["Testing Chat Frame Printing"])
	end
	if self.db.profile.showOnScreen then
		UIErrorsFrame:AddMessage(L["Testing On Screen Printing"], self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
	end
	if self.db.profile.showInChannel and channel ~= "" then
		SendChatMessage(L["Testing Channel Printing"], "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
	end
	if self.db.profile.showFloating then
		self:Pour(L["Testing Floating Message"], self.db.profile.r, self.db.profile.g, self.db.profile.b)
	end
end

function SimpleMD:CreateAnchor(text, r, g, b)
	local f = CreateFrame("BUTTON", nil, UIParent)
	f:SetWidth(200)
	f:SetHeight(25)
	
	f.owner = self
	
	if self.db.profile.position.x and self.db.profile.position.y then
		f:ClearAllPoints()
		f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.db.profile.position.x, self.db.profile.position.y)
	else
		f:SetPoint("CENTER", UIParent, "CENTER")
	end
	
	f:SetScript("OnDragStart", function() this:StartMoving() end )
	f:SetScript("OnDragStop",
		function ()
			this:StopMovingOrSizing()
			local _, _, _, x, y = this:GetPoint()
			this.owner.db.profile.position.x = math.floor(x)
			this.owner.db.profile.position.y = math.floor(y)
		end)
	
	f:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = false, tileSize = 16, edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 }
		})
	f:SetBackdropColor(r, g, b, .6)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")
	f:SetClampedToScreen(true)
	f:EnableMouse(true)
	
	
	f.text = f:CreateFontString(nil, "OVERLAY")
	f.text:SetFontObject(GameFontNormalSmall)
	f.text:ClearAllPoints()
	f.text:SetTextColor(1, 1, 1, 1)
	f.text:SetWidth(200)
	f.text:SetHeight(25)
	f.text:SetPoint("TOPLEFT", f, "TOPLEFT")
	f.text:SetJustifyH("CENTER")
	f.text:SetJustifyV("MIDDLE")
	f.text:SetText(text)
	
	f:Hide()
	
	return f
end

function SimpleMD:ToggleAnchor()
	if self.anchor:IsVisible() then
		self.anchor:Hide()
	else
		self.anchor:Show()
	end
end

function SimpleMD:GetTexture(name)
	local i = 1
	while true do
		local spell = GetSpellName(i, BOOKTYPE_SPELL)
		if not spell then break
		elseif spell == name then
			tex = GetSpellTexture(i, BOOKTYPE_SPELL)
			return tex
		end
		i = i + 1
	end
end

function SimpleMD:StartTimer(name)
	local id = name.."1"
	
	for i = 20, 1, -1 do
		if self.bars[i].id == id then
			self:SetCandyBarFade(id, 0, false)
			self:StopCandyBar(id)
			self:StopBar(id)
			break
		end
	end
	
	local slot = 0
	for i = 1, 20 do
		if not self.bars[i].id then 
			slot = i
			break
		end
	end
	
	self.bars[slot].id = id
	self.bars[slot].name = name
	
	local text = name
	local barColor = self.db.profile.bars
		
	self:RegisterCandyBar(id, 120, text, self.tex, barColor, barColor, barColor, "green")
	self:RegisterCandyBarWithGroup(id, "SimpleMD")
	if self.db.profile.texture then self:SetCandyBarTexture(id, self.textures[self.db.profile.texture]) end
	if self.db.profile.bg then self:SetCandyBarBackgroundColor(id, self.db.profile.bg, self.db.profile.bg_alpha) end
	if self.db.profile.textColor then self:SetCandyBarTextColor(id, self.db.profile.txtColor) end
	if self.db.profile.textSize then self:SetCandyBarFontSize(id, self.db.profile.textSize) end
	if self.db.profile.barScale then self:SetCandyBarScale(id, self.db.profile.barScale) end
	if self.db.profile.barWidth then self:SetCandyBarWidth(id, self.db.profile.barWidth) end
	if self.db.profile.barHeight then self:SetCandyBarHeight(id, self.db.profile.barHeight) end
	self:SetCandyBarReversed(id, self.db.profile.reverse)
	self:SetCandyBarFade(id, 2, true)
	self:SetCandyBarCompletion(id, self.StopBar, self, id)
	self:StartCandyBar(id, true)
end

function SimpleMD:StopBar(id)
	for i = 1, 20 do
		if self.bars[i].id == id then
			self.bars[i] = {}
		end
	end
	
	for i = 1, 19 do
		if not self.bars[i].id then
			local temp = self.bars[i]
			for j = i + 1, 20 do
				if self.bars[j].id then
					self.bars[i] = self.bars[j]; self.bars[j] = temp; temp = nil
					break
				end
			end
			if temp then break end
		end
	end
end

function SimpleMD:DisableBars()
	if self.db.profile.showBars then
		for i = 1, 20 do
			if self.bars[i].id then
				self:StopCandyBar(self.bars[i].id)
			end
		end
	end
	self.db.profile.showBars = not self.db.profile.showBars
end

function SimpleMD:RemoveBars()
	if self.db.profile.showBars then
		for i = 1, 20 do
			if self.bars[i].id then
				self:StopCandyBar(self.bars[i].id)
			end
		end
	end
end

--[[function SimpleMD:BarTest()
	self:RemoveBars()
	self:StartTimer("Test1")
	self:StartTimer("test2")
end]]

function SimpleMD:UpdateColors()
	self.count = self.count + 1
	local i = tostring(self.count)
	colors:RegisterColor("userBar"..i, self.db.profile.barR, self.db.profile.barG, self.db.profile.barB)
	colors:RegisterColor("userBG"..i, self.db.profile.BGr, self.db.profile.BGg, self.db.profile.BGb)
	colors:RegisterColor("userText"..i, self.db.profile.txtR, self.db.profile.txtG, self.db.profile.txtB)
	self.db.profile.txtColor = "userText"..i
	self.db.profile.bg = "userBG"..i
	self.db.profile.bars = "userBar"..i
end
